
/******************************************************************************
*@file  : main.c
*@brief : main program
*@ver   : 1.0.0
*@date  : 2022.10.20
******************************************************************************/

#include "main.h" 
#include "app.h"  
#include "hal.h" 

//mpu region config paramters,max 8 regions
//region can not overlap
const MPU_RegionConfigTypeDef MPU_regionConfigs[]=
{
   //FLASH/NVR/ROM Space,  cacheable,write-through , code
   //FLASH MAIN:0x00000000~(0x00050000-1)
   //FLASH NVR: 0x00080000~(0x00081000-1) 
   //ROM:       0x12000000~(0x12003000-1)
   {
       0x00000000,
       0x20000000-1, 
       MPU_ACCESS_ALL_READ_WRITE, 
       MPU_EXECUTE_ENABLE, 
       MPU_ATTR_WRITE_THOUGH,
   },
  
   //SRAM lower 32KB, cacheable,write-back, data  
   {
       0x20000000,
       0x20008000-1, 
       MPU_ACCESS_ALL_READ_WRITE,
       MPU_EXECUTE_ENABLE,  
       MPU_ATTR_WRITE_BACK,
   }, 
   
   //SRAM upper 32KB, non-cacheable, data  
   {
       0x20008000,
       0x20010000-1,
       MPU_ACCESS_ALL_READ_WRITE,
       MPU_EXECUTE_DISABLE,  
       MPU_ATTR_NO_CACHE,
   }, 
   
   //Device memory type          
   {
       0x40000000,
       0xFFFFFFFF, 
       MPU_ACCESS_ALL_READ_WRITE,
       MPU_EXECUTE_DISABLE,  
       MPU_ATTR_DEVICE,
   },
};


void MPU_PrintReg(void)
{
    uint32_t i,index,attr,mair;
    printfS("CTRL:%08x\r\n",MPU->CTRL);
    for(i=0;i<8;i++)
    {
        MPU->RNR = i;
        printfS("%d,RBAR:%08x,RLAR:%08x,",i,MPU->RBAR,MPU->RLAR);
        index = (MPU->RLAR & MPU_RLAR_AttrIndx_Msk)>>MPU_RLAR_AttrIndx_Pos;
        printfS("ATTR%d:",index);
        if(index<4)
            mair = MPU->MAIR0;
        else
        {
            mair = MPU->MAIR1; 
            index -= 4;
        }
        attr = (mair>>(index*8)) & 0xff;
        
        printfS("%02x\r\n",attr);  
    }
    
    printfS("MAIR0:%08x,MAIR1:%08x\r\n",MPU->MAIR0,MPU->MAIR1); 
}

void MPU_Config(void)
{
    MPU_ConfigInitTypeDef mpuConfigInit;
    
    mpuConfigInit.HfNmi = MPU_HFNMI_ENABLE;
    mpuConfigInit.PrivDef = MPU_PRIVDEF_DISABLE; 
    mpuConfigInit.RegionCount = sizeof(MPU_regionConfigs) / sizeof(MPU_regionConfigs[0]);
    mpuConfigInit.RegionConfigs = (MPU_RegionConfigTypeDef *)MPU_regionConfigs;
    
    HAL_MPU_Config(&mpuConfigInit); 
}

/******************************************************************************
*@brief : redefine HAL_MspInit, config mpu
*@param : none
*@return: none
******************************************************************************/
void HAL_MspInit(void)
{
    MPU_Config();    
}



/******************************************************************************
*@brief : main program
*@param : none
*@return: none
******************************************************************************/
int main(void)
{
    int count=0;
	HAL_Init();  
    SystemClock_Config();    
    BSP_UART_Init();  
    
//    if(HAL_EFLASH_EraseSecurityCode()==HAL_OK)
//    {
//        printfS("ȫгɹ\r\n");
//    }
//    else
//    {
//        printfS("ȫʧ\r\n");        
//    }
 
        
	
    count=5;
     while(count--)
    {
        APP_Test();
        HAL_Delay(1000);
        printfS("idle %d\r\n",count);
    }  
    
    //RCC->RCR &= (~RCC_RCR_EFCRST);
    
    count=0;
    while(1)
    {
        HAL_Delay(2000);
        printfS("idle %d\r\n",count++);
    }
}


